<?php if ( ! defined( 'EVENT_ESPRESSO_VERSION' ) ) { exit('No direct script access allowed'); }

/**
 *
 * EEG_AuthorizeNet_Sim
 *
 * @package			Event Espresso
 * @subpackage		eea-authorizenet-sim
 * @author			Event Espresso
 *
 */

class EEG_AuthorizeNet_Sim extends EE_Offsite_Gateway {

	/**
	 *
	 * @var $_AuthorizeNet_Sim_transaction_key string.
	 */
	protected $_transaction_key = null;


	/**
	 *
	 * @var $_AuthorizeNet_Sim_api_login_id string.
	 */
	protected $_api_login_id = null;


	/**
	 *
	 * @var $_test_transactions_yn string
	 */
	protected $_test_transactions_yn = null;


	/**
	 *
	 * @var $_gateway_url string
	 */
	protected $_gateway_url = null;


	protected $_currencies_supported = array(
		'USD',
		'CAD',
		'AUD',
		'EUR',
		'GBP',
		'NZD'
	);


	/**
	 * Sets the gateway url variable based on whether debug mode is enabled or not.
	 *
	 * @param type $settings_array
	 */
	public function set_settings( $settings_array ) {
		parent::set_settings($settings_array);
		if ( $this->_debug_mode ) {
			$this->_gateway_url = 'https://test.authorize.net/gateway/transact.dll';
		} else {
			$this->_gateway_url = 'https://secure.authorize.net/gateway/transact.dll';
		}
	}

	/**
	 * Override parent so we can declare that we receive an IPN-request (the relay resposne)
	 * and so shouldn't process payment-stuff when returning to SPCO.
	 * Also declare we don't support receiving notifications via refund yet
	 */
	public function __construct() {
		parent::__construct();
		//only use the method set_uses_separate_IPN_request if the method exists,
		//which was in 4.6.18 or so
		if( method_exists( $this, 'set_uses_separate_IPN_request' ) ) {
			$this->set_uses_separate_IPN_request( true );
		}
		$this->_supports_receiving_refunds = false;
	}


	/**
	 *
	 * @param EEI_Payment $payment      to process
	 * @param array       $billing_info but should be empty for this gateway
	 * @param string      $return_url   URL to send the user to after payment on the payment provider's website
	 * @param string      $notify_url   URL to send the instant payment notification
	 * @param string      $cancel_url   URL to send the user to after a cancelled payment attempt on teh payment provider's website
	 * @return EEI_Payment
	 */
	public function set_redirection_info( $payment, $billing_info = array(), $return_url = null, $notify_url = null, $cancel_url = null ) {
		$redirect_args = array();
		$test_request = FALSE;
		$transaction = $payment->transaction();
		$primary_registrant = $transaction->primary_registration();
		$primary_attendee = ( $primary_registrant instanceof EEI_Registration ) ? $primary_registrant->attendee() : null;
		$order_description  = $this->_format_order_description( $payment );

		$amount = number_format( $payment->amount(), 2, '.', '' );
		// The invoice is generated using the date and time.
		$invoice = date( 'YmdHis' );
		// Randomly generate the sequence number.
		$sequence = rand( 1, 1000 );
		// Generate the timestamp.
		$timeStamp = time();
		// Generate the SIM fingerprint.
		if ( phpversion() >= '5.1.2' ) {
			$fingerprint = hash_hmac( 'md5', $this->_api_login_id . '^' . $sequence . '^' . $timeStamp . '^' . $amount . '^' . $payment->currency_code(), $this->_transaction_key );
		} else {
			$fingerprint = bin2hex( mhash( MHASH_MD5, $this->_api_login_id . '^' . $sequence . '^' . $timeStamp . '^' . $amount . '^' . $payment->currency_code(), $this->_transaction_key ) );
		}

		if ( $this->_test_transactions_yn ) {
			$test_request = TRUE;
		}
		//if in debug mode, use authorize.net's sandbox parnter id; otherwise use the Event Espresso partner id 
		$redirect_args['x_solution_id'] = $this->_debug_mode ? 'AAA100302' : 'AAA105363';
		$redirect_args['x_version'] = '3.1';
		$redirect_args['x_login'] = $this->_api_login_id;
		$redirect_args['x_fp_sequence'] = $sequence;
		$redirect_args['x_fp_timestamp'] = $timeStamp;
		$redirect_args['x_fp_hash'] = $fingerprint;
		
		if ( $primary_attendee instanceof EE_Attendee ) {
			$redirect_args['x_first_name'] = substr($primary_attendee->fname(), 0, 50);
			$redirect_args['x_last_name'] = substr($primary_attendee->lname(), 0, 50);
			$redirect_args['x_email'] = substr($primary_attendee->email(), 0, 255);
			
			$redirect_args['x_address'] = ($primary_attendee->address() && strlen($primary_attendee->address()) > 0) ? substr($primary_attendee->address(), 0, 60) : '';
			$redirect_args['x_city'] = ($primary_attendee->city() && strlen($primary_attendee->city()) > 0) ? substr($primary_attendee->city(), 0, 40) : '';
			$redirect_args['x_state'] = ($primary_attendee->state_name() && strlen($primary_attendee->state_name()) > 0) ? substr($primary_attendee->state_name(), 0, 40) : '';
			$redirect_args['x_country'] = ($primary_attendee->country_name() && strlen($primary_attendee->country_name()) > 0) ? substr($primary_attendee->country_name(), 0, 60) : '';
			$redirect_args['x_zip'] = ($primary_attendee->zip() && strlen($primary_attendee->zip()) > 0) ? substr($primary_attendee->zip(), 0, 20) : '';
			$redirect_args['x_phone'] = ($primary_attendee->phone() && strlen($primary_attendee->phone()) > 0) ? substr($primary_attendee->phone(), 0, 25) : '';
		}

		$redirect_args['x_amount'] = $amount;
		$redirect_args['x_currency_code'] = $payment->currency_code();
		$redirect_args['x_type'] = 'auth_capture';
		$redirect_args['x_method'] = 'CC';
		$redirect_args['x_description'] = $order_description;
		$redirect_args['x_invoice_num'] = $invoice;

		$redirect_args['x_show_form'] = 'PAYMENT_FORM';
		$redirect_args['x_test_request'] = $test_request;

		// Using the Relay Response Receipt method.
		$redirect_args['x_relay_response'] = TRUE;
		$redirect_args['x_relay_always'] = TRUE;
		$redirect_args['x_relay_url'] = $notify_url;

		$redirect_args['x_cancel_url'] = $cancel_url;
		$redirect_args['x_cancel_url_text'] = 'Cancel Payment';
		$redirect_args['transaction_return_url'] = $return_url . '&sim_action=transaction_return';

		$payment->set_redirect_url( $this->_gateway_url );
		$payment->set_redirect_args( $redirect_args );
		$this->log( array('AuthNet SIM Request' => $redirect_args), $payment );

		return $payment;
	}


	/**
	 * Handles the payment update.
	 *
	 * Note: Authorize.net SIM uses a 'Relay Response' for communicating the
	 * transaction results to the customer which does not redirect the customer back to the merchant website,
	 * but it relays the content from the specified Relay URL (from the merchant's website) to the customer through Authorize.net receipt page.
	 * So we use a script to redirect the customer back to the purchase website as a response to that Relay call.
	 *
	 * @param array $update_info like $_POST
	 * @param EEI_Transaction $transaction
	 * @return EE_Payment updated
	 * @throws EE_Error
	 */
	public function handle_payment_update( $update_info, $transaction ) {
		$payment = $transaction instanceof EEI_Transaction ? $transaction->last_payment() : null;

		// If this is a "Thank you page update" (just a transaction return) do not handle the payment update.
		if ( isset($update_info['sim_action']) && $update_info['sim_action'] == 'transaction_return' ) {
			// We got here through the 'SIM Relay Response' (and our redirect script) so the payment should've been updated by now.
			return $payment;
		}

		if ( $payment && $payment instanceof EEI_Payment ) {
			if ( isset($update_info['x_response_code']) ) {
				switch ( $update_info['x_response_code'] ) {
					case '1':
						$payment->set_status( $this->_pay_model->approved_status() );
						$payment->set_amount( floatval($update_info['x_amount']) );
						break;
					case '4':
						$payment->set_status( $this->_pay_model->pending_status() );
						break;
					case '2':
						$payment->set_status( $this->_pay_model->declined_status() );
						break;
					case '3':
						$payment->set_status( $this->_pay_model->failed_status() );
						break;
					default:
						$payment->set_status( $this->_pay_model->declined_status() );
				}
				$payment->set_gateway_response( $update_info['x_response_reason_text'] );
				$payment->set_txn_id_chq_nmbr( $update_info['x_trans_id'] );
				$this->log( array('AuthNet SIM Response' => $update_info), $payment );
			} else {
				$payment->set_gateway_response( __( 'The response from Authorize.Net could to be defined.', 'event_espresso' ) );
				$payment->set_status( $this->_pay_model->failed_status() );
				$this->log( array('SIM response could to be defined' => $update_info), $payment );
			}

			// On 'Relay Response' redirect the user back to the merchant website.
			add_filter( 'FHEE__EES_Espresso_Txn_Page__run__exit', function( $do_exit ) use ( $update_info ) {
				$blog_title = get_bloginfo( 'name' );
				$return_template = file_get_contents( dirname(__FILE__) . DS . 'templates' . DS . 'return_page.template.php' );
				$return_template = preg_replace("/%%return_url%%/", $update_info['transaction_return_url'], $return_template );
				$return_template = preg_replace("/%%website_name%%/", $blog_title, $return_template );
				echo $return_template;
				return true;
			} );

			return $payment;
		} else {
			$this->log( array('No payment for the transaction' => $update_info), $payment );
			throw new EE_Error( sprintf(__("Could not find Authorize.Net SIM payment for transaction %s", 'event_espresso'), $transaction->ID()) );
		}
	}

}

// End of file EEG_AuthorizeNet_Sim.gateway.php
