<?php

if (!defined('EVENT_ESPRESSO_VERSION')) {
    exit('No direct script access allowed');
}



/**
 *
 * Class EE_PMT_Vanco_Onsite
 *
 *
 * @package			Event Espresso
 * @subpackage		eea-vanco-payment-method
 * @author			Event Espresso
 *
 */
class EE_PMT_Vanco_Onsite extends EE_PMT_Base
{

    /**
     * 	Path to the templates folder for the EE Vanco PM.
     * 	@var string
     */
    protected $_template_path = null;

    /**
     * 	Vanco allowed account types list.
     * 	@var array
     */
    public $_account_types = array(
        'CC' => 'Credit or Debit Card',
        'C'  => 'Checking',
        'S'  => 'Savings'
    );


    /**
     * 	Class constructor.
     *
     * 	@param EE_Payment_Method $pm_instance
     * 	@throws \EE_Error
     * 	@return \EE_PMT_Vanco_Onsite
     */
    public function __construct($pm_instance = null)
    {
        $this->_pretty_name = esc_html__("Vanco", 'event_espresso');
        $this->_default_description = esc_html__('Please provide the following billing information.', 'event_espresso');
        $this->_template_path = dirname(__FILE__) . DS . 'templates' . DS;
        $this->_requires_https = true;
        require_once($this->file_folder() . 'EEG_Vanco_Onsite.gateway.php');
        $this->_gateway = new EEG_Vanco_Onsite();

        parent::__construct($pm_instance);
        // Vanco Login request Cron.
        $this->schedule_vanco_session_update();
        // Vanco PM Scripts.
        add_action('wp_enqueue_scripts', array($this, 'enqueue_vanco_payment_scripts'));
    }


    /**
     * 	Generate a new payment settings form.
     *
     * 	@return EE_Payment_Method_Form
     */
    public function generate_new_settings_form()
    {
        return new EE_Payment_Method_Form(array(
            'extra_meta_inputs' => array(
                'client_id'         => new EE_Text_Input(
                    array(
                        'html_label_text' => sprintf(
                            esc_html__("Vanco Client ID %s", "event_espresso"),
                            $this->get_help_tab_link()
                        ),
                        'required'        => true
                    )
                ),
                'user_id'           => new EE_Text_Input(
                    array(
                        'html_label_text' => sprintf(
                            esc_html__("Vanco User ID %s", "event_espresso"),
                            $this->get_help_tab_link()
                        ),
                        'required'        => true
                    )
                ),
                'user_password'     => new EE_Password_Input(
                    array(
                        'html_label_text' => sprintf(
                            esc_html__("Vanco Password %s", "event_espresso"),
                            $this->get_help_tab_link()
                        ),
                        'required'        => true
                    )
                ),
                'encryption_key'    => new EE_Text_Input(
                    array(
                        'html_label_text' => sprintf(
                            esc_html__("Your unique Vanco Encryption Key %s", "event_espresso"),
                            $this->get_help_tab_link()
                        ),
                        'required'        => true
                    )
                ),
                'is_debitcard_only' => new EE_Yes_No_Input(
                    array(
                        'html_label_text' => sprintf(
                            esc_html__("Allow only Debit Cards %s", "event_espresso"),
                            $this->get_help_tab_link()
                        ),
                        'required'        => true,
                        'default'         => false
                    )
                )
            )
        ));
    }


    /**
     * 	Creates a billing form for this payment method type.
     *
     * 	@param \EE_Transaction $transaction
     * 	@return \EE_Billing_Info_Form
     */
    public function generate_new_billing_form(EE_Transaction $transaction = null)
    {
        require_once($this->file_folder() . 'EEA_Vanco_Billing_Form.form.php');
        $form = new EEA_Vanco_Billing_Form($this, $this->_pm_instance);
        return $this->apply_billing_form_debug_settings($form);
    }


    /**
     * 	Get all active US and Canada states.
     *
     * 	@return array
     */
    public function get_state_abbrv_list()
    {
        $state_options = array();
        $states = EEM_State::instance()->get_all_active_states();
        if (!empty($states)) {
            foreach ($states as $state) {
                if ($state instanceof EE_State) {
                    $state_options[$state->country()->name()][$state->abbrev()] = $state->name();
                }
            }
        } else {
            $state_options = array();
        }
        return $state_options;
    }


    /**
     * 	Possibly adds debug content to Vanco checkout billing form.
     *
     * 	@param \EE_Billing_Info_Form $billing_form
     * 	@return \EE_Billing_Info_Form
     */
    public function apply_billing_form_debug_settings(EE_Billing_Info_Form $billing_form)
    {
        if ($this->_pm_instance->debug_mode()) {
            $billing_form->add_subsections(
                array(
                    'fyi_about_autofill' => $billing_form->payment_fields_autofilled_notice_html()
                ),
                'account_type'
            );
            $billing_form->add_subsections(
                array(
                    'debug_content' => new EE_Form_Section_HTML_From_Template(
                        $this->_template_path . 'vanco_debug_info.template.php'
                    )
                ),
                'first_name'
            );
            $billing_form->get_input('credit_card')->set_default('4012000033330026');
            $billing_form->get_input('routing_number')->set_default('123456780');
            $billing_form->get_input('bank_account')->set_default('6011000991300009');
            $billing_form->get_input('exp_month')->set_default(date('m'));
            $billing_form->get_input('exp_year')->set_default(date('Y') + 6);
            $billing_form->get_input('name_on_card')->set_default('Test Customer');
        }
        return $billing_form;
    }


    /**
     *  Load all the scripts needed for the Vanco checkout.
     *
     * @return void
     * @throws \EE_Error
     */
    public function enqueue_vanco_payment_scripts()
    {
        // Add a list of Vanco error codes.
        $error_mapping = EEH_File::get_file_contents(EEA_VANCO_PM_PATH . 'error-code-mapping.json');
        $vanco_error_list = json_decode($error_mapping, true);

        wp_enqueue_script(
            'eea_vanco_payment_js',
            EEA_VANCO_PM_URL . 'assets' . DS . 'js' . DS . 'eea_vanco_payment_method.js',
            array('single_page_checkout'),
            EEA_VANCO_PM_VERSION,
            true
        );

        // Data needed in the JS.
        $transaction_args = array(
            'requesturl'                 => $this->_pm_instance->debug_mode()
                                                ? 'https://uat.vancopayments.com/cgi-bin/wsnvp.vps'
                                                : 'https://myvanco.vancopayments.com/cgi-bin/wsnvp.vps',
            // @codingStandardsIgnoreStart
            'jsonp_error_message'        => esc_html__('Payment Error! Please refresh the page and try again or contact support.',
                'event_espresso'),
            'accepted_message'           => esc_html__('Your Information was Accepted. Vanco created a Transaction. Click "Finalize Registration" to proceed.',
                'event_espresso'),
            'no_SPCO_error'              => esc_html__('It appears the Single Page Checkout javascript was not loaded properly! Please refresh the page and try again or contact support.',
                'event_espresso'),
            'no_NVPVAR_error'            => esc_html__('It appears that Vanco Session ID is missing! Please refresh the page and try again or contact support.',
                'event_espresso'),
            'no_sessionid_error'         => esc_html__('Error while getting a session ID.', 'event_espresso'),
            'invalid_SPCO_submit_button' => esc_html__('The Registration Form Submit button could not be located! Please refresh the page and try again or contact support.',
                'event_espresso'),
            'error_list'                 => $vanco_error_list
            // @codingStandardsIgnoreEnd
        );

        // Localize the script with our transaction data.
        wp_localize_script('eea_vanco_payment_js', 'EEA_VANCO_ARGS', $transaction_args);
    }


    /**
     * 	Adds the help tab
     *
     * 	@see EE_PMT_Base::help_tabs_config()
     * 	@return array
     */
    public function help_tabs_config()
    {
        return array(
            $this->get_help_tab_name() => array(
                'title'    => esc_html__('Vanco Settings', 'event_espresso'),
                'filename' => 'payment_methods_overview_vanco'
            )
        );
    }


    /**
     *    Schedule the Vanco Login request.
     *
     * @return void
     */
    public function schedule_vanco_session_update()
    {
        $cron_hook = 'eea_vanco_session_id_update';
        if (!wp_next_scheduled($cron_hook)) {
            // Should update every 24 hours.
            wp_schedule_event(time(), 'daily', $cron_hook);
        }
        add_action($cron_hook, array('EE_PMT_Vanco_Onsite', 'vanco_login_request'));
    }


    /**
     *    Return the encrypted nvpvar (on AJAX call).
     *
     * @return void
     */
    public static function ajax_encrypt_nvpvar()
    {
        $vanco_pm = EEM_Payment_method::instance()->get_one_of_type('Vanco_Onsite');
        $sessionid = $vanco_pm->get_extra_meta('vanco_session_id', true);
        $session_updated = $vanco_pm->get_extra_meta('vanco_session_id_updated', true);
        $timediff = time() - $session_updated;
        $client_id = array('client_id' => $vanco_pm->get_extra_meta('user_id', true));
        $debitcard_only = 'No';
        $session_inf = '';
        if ($vanco_pm->get_extra_meta('is_debitcard_only', true)) {
            $debitcard_only = 'Yes';
        }
        $nvpvar = $_POST['nvpvar'] . '&isdebitcardonly=' . $debitcard_only;
        // Update the Session ID if not present or maybe deprecated (will update every 20 hours [72000]).
        if (!$sessionid || empty($sessionid) || ( $timediff >= 72000 )) {
            $vanco_session = EE_PMT_Vanco_Onsite::vanco_login_request();
            $sessionid = $vanco_session['sessionid'];
            $session_inf = $vanco_session['session_inf'];
        }
        $nvp_var = EE_PMT_Vanco_Onsite::vanco_encrypt_nvpvar($nvpvar);
        $vanco_pm->type_obj()->get_gateway()->log(
            array(
                'Client:' => $client_id,
                'NVPVAR:' => $nvpvar
            ),
            null
        );
        wp_send_json(
            array(
                'nvp_var'     => $nvp_var,
                'session_id'  => $sessionid,
                'session_inf' => $session_inf
            )
        );
    }


    /**
     *    Login Service API
     *    Do a Vanco login request to obtain the Session ID for performing other services.
     *
     * @return array ('sessionid', 'session_inf')  Vanco Session ID and some extra information
     */
    public static function vanco_login_request()
    {
        $vanco_pm = EEM_Payment_method::instance()->get_one_of_type('Vanco_Onsite');
        $login_url = $vanco_pm->debug_mode()
            ? 'https://uat.vancopayments.com/cgi-bin/wsnvp.vps'
            : 'https://myvanco.vancopayments.com/cgi-bin/wsnvp.vps';
        $requestid = date('YmdHis') . '' . rand(0, 9999);
        $user_id = $vanco_pm->get_extra_meta('user_id', true);
        $user_pass = $vanco_pm->get_extra_meta('user_password', true);
        $args = array(
            'method'      => 'POST',
            'timeout'     => 60,
            'redirection' => 5,
            'blocking'    => true,
            'headers'     => array(
                'Content-Type' => 'application/x-www-form-urlencoded'
            ),
            'body'        => 'nvpvar=requesttype=login&userid=' . $user_id . '&password=' . $user_pass . '&requestid=' . $requestid,
            'cookies'     => array()
        );

        $vanco_login = wp_remote_post($login_url, $args);
        // Log the response.
        $vanco_pm->type_obj()->get_gateway()->log(array('Vanco Login Request Response:' => $vanco_login), null);
        if (!is_wp_error($vanco_login) && (strpos($vanco_login['body'], 'nvpvar') !== false)) {
            $nvpvar = $vanco_login['body'];
            if (strpos($nvpvar, 'errorlist') !== false) {
                $session_inf = esc_html__('Session request Error', 'event_espresso');
                // Try to get some details for this error by it's code.
                $nvp_args = array();
                parse_str(str_replace('nvpvar=', '', $nvpvar), $nvp_args);
                if (!empty($nvp_args['errorlist'])) {
                    $error_list = explode(',', $nvp_args['errorlist']);
                    foreach ($error_list as $error_code) {
                        $session_inf .= ' : ' . EE_PMT_Vanco_Onsite::get_vanco_error_description($error_code);
                    }
                }
                $vanco_pm->update_extra_meta('vanco_session_id', false);
                $vanco_pm->type_obj()->get_gateway()->log(
                    array(
                        'Vanco Login Request Failed' => $nvpvar,
                        'Vanco Login args:'          => 'nvpvar=requesttype=login&userid=' . $user_id
                                                            . '&password=******&requestid=' . $requestid
                    ),
                    null
                );
                $sessionid = false;
            } else {
                $dec_nvpvar = EE_PMT_Vanco_Onsite::vanco_decrypt_nvpvar(str_replace('nvpvar=', '', $nvpvar));
                $dec_nvparray = array();
                parse_str($dec_nvpvar, $dec_nvparray);
                $vanco_pm->update_extra_meta('vanco_session_id', $dec_nvparray['sessionid']);
                $vanco_pm->update_extra_meta('vanco_session_id_updated', time());
                $vanco_pm->type_obj()->get_gateway()->log(
                    array(
                        'Vanco Login:'      => 'OK',
                        'Vanco Login args:' => 'nvpvar=requesttype=login&userid=' . $user_id
                                                    . '&password=******&requestid=' . $requestid
                    ),
                    null
                );
                $sessionid = $dec_nvparray['sessionid'];
                $session_inf = 'OK';
            }
        } else {
            //$vanco_login is either a wp_error object or nvpvar was not returned.
            $vanco_pm->type_obj()->get_gateway()->log(
                array(
                    'Vanco Login Failed:' => 'Login request returned WP_Error.',
                    'Login response:'     => $vanco_login
                ),
                null
            );

            $sessionid = false;
            $session_inf = '';
        }
        return array('sessionid' => $sessionid, 'session_inf' => $session_inf);
    }


    /**
     *    Do a Vanco login request and return the session ID as a response.
     *
     * @return void
     */
    public static function vanco_ajax_login_request()
    {
        $vanco_session = EE_PMT_Vanco_Onsite::vanco_login_request();
        $sessionid = $vanco_session['sessionid'];
        $session_inf = $vanco_session['session_inf'];
        wp_send_json(
            array(
                'session_id'  => $sessionid,
                'session_inf' => $session_inf
            )
        );
    }


    /**
     *    Decrypt Vanco 'nvpvar'.
     *
     * @param string $nvpvar Encrypted response.
     * @return array    Decrypted 'nvpvar' variables.
     */
    public static function vanco_decrypt_nvpvar($nvpvar)
    {
        $vanco_pm = EEM_Payment_method::instance()->get_one_of_type('Vanco_Onsite');
        $encryption_key = $vanco_pm->get_extra_meta('encryption_key', true);
        $b64_decoded = base64_decode(str_pad(strtr($nvpvar, '-_', '+/'), strlen($nvpvar) % 4, '=', STR_PAD_RIGHT));
        $decrypted = openssl_decrypt($b64_decoded, 'aes-256-ecb', $encryption_key, OPENSSL_RAW_DATA | OPENSSL_ZERO_PADDING);
        $inflated = gzinflate($decrypted);
        return $inflated;
    }


    /**
     *    Encrypt Vanco 'nvpvar'.
     *
     * @param array $nvpvar Request string.
     * @return string    Encrypted 'nvpvar' variables.
     */
    public static function vanco_encrypt_nvpvar($nvpvar)
    {
        $vanco_pm = EEM_Payment_method::instance()->get_one_of_type('Vanco_Onsite');
        $encryption_key = $vanco_pm->get_extra_meta('encryption_key', true);
        $client_id = $vanco_pm->get_extra_meta('client_id', true);
        $request_id = date('YmdHis') . '' . rand(0, 9999);
        $plain_nvp = $nvpvar . '&requestid=' . $request_id . '&clientid=' . $client_id;
        $deflated = gzdeflate($plain_nvp);
        $padded = $deflated . str_repeat(' ', strlen($deflated) % 16);
        $encrypted = openssl_encrypt($padded, 'aes-256-ecb', $encryption_key, OPENSSL_RAW_DATA);
        $encoded = rtrim(strtr(base64_encode($encrypted), '+/', '-_'));
        return $encoded;
    }


    /**
     *    Log TXN Error.
     *
     * @return void
     */
    public static function vanco_log_error()
    {
        if (isset($_POST['txn_id']) && !empty($_POST['txn_id'])) {
            $vanco_pm = EEM_Payment_method::instance()->get_one_of_type('Vanco_Onsite');
            $transaction = EEM_Transaction::instance()->get_one_by_ID($_POST['txn_id']);
            $vanco_pm->type_obj()->get_gateway()->log(
                array(
                    'JS Error (Transaction: ' . $transaction->ID() . ')' => $_POST['message']
                ),
                $transaction
            );
        }
    }


    /**
     *    Returns an error description by the code, if present.
     *
     * @param string $error_code Error id.
     * @return string    Error description.
     * @throws \EE_Error
     */
    public static function get_vanco_error_description($error_code)
    {
        $error_mapping = EEH_File::get_file_contents(EEA_VANCO_PM_PATH . 'error-code-mapping.json');
        $error_list = json_decode($error_mapping, true);

        return (isset($error_list[$error_code])) ? $error_list[$error_code] : $error_code;
    }
}

// End of class EE_PMT_Vanco_Onsite.
// End of file EE_PMT_Vanco_Onsite.pm.php